dat = read.table("Demographic_rates.txt", header=T)

dat = dat[dat$site == "CostaRica",]

# demographic rates in canopy layers 1-3 and recruitment rate
X = cbind(dat$survival1.trans, dat$survival2.trans, dat$survival3.trans,
          dat$growth1.trans, dat$growth2.trans, dat$growth3.trans, dat$recruitment.trans)
# weights for demographic rates
W = cbind(dat$survival1.weight, dat$survival2.weight, dat$survival3.weight, 
          dat$growth1.weight, dat$growth2.weight, dat$growth3.weight, dat$recruitment.weight)


Xp <- t(as.matrix(X))
Wp <- t(as.matrix(W))

# if more demographic rates are included in the PCA, e.g. when 4 canopy layers are included, the number of dimensions has to be adjusted
PCAres = wPCA(Xp,Wp,7)

rownames(PCAres$loadings) = c("Survival 1","Survival 2","Survival 3","Growth 1","Growth 2","Growth 3","Recruitment")
# laodings of demographic rates on principal components; the direction of loadings (signs) on each is arbitrary 
PCAres$loadings

# coordinates of species (rows) in PCA space (columns)
head(PCAres$coords)


################################################
######            weighted PCA             #####
################################################

wPCA <- function(X, W, rankn)
{
    # implemented by Benjamin Rosenbaum, German Centre for Integrative Biodiversity Research (iDiv) Halle-Jena-Leipzig (benjamin.rosenbaum@idiv.de) 
    #  
    # Compute a weighted PCA by eigenvalue decomposition of the weighted covariance matrix
    #
    # see L. Delchambre. Weighted principal component analysis: a weighted covariance eigendecomposition approach. Mon. Not. R. Astron. Soc. 446(2), 3545-3555, 2014. 
    #
    # Args:
    #   X: matrix of observations. rows=variables, cols=observations (transpose data frame)
    #   W: corresponding weights
    #
    # Returns:
    #   Chi2: explained variance
    #   P:    matrix of principal components (new basis)
    #   C:    matrix of coefficients (coordinates in new basis)
    #   PC:   matrix of projections of old data (PC approx X)
    
    #----------------------------------------------------------------
    # step 1: read data and compute EV of weighted covariance matrix
    #----------------------------------------------------------------
    
    W[W==0] <- 1.0e-6
    
    n <- as.numeric(ncol(X))
    d <- as.numeric(nrow(X))
    
    # substract weighted mean, save original data
    Xorig <- X
    X <- X - rowMeans(W*X)/rowMeans(W)
    # rowMeans(W*X) < 1.0e-16
    
    # scale by weighted sd in each dimension
    for (i in 1:d){
      sd <- sqrt(sum(W[i, ]^2*X[i, ]^2)/sum(W[i, ]^2))
      X[i, ] <- X[i, ]/sd
    }
    
    # weighted covariance matrix S
    S <- (W*X)%*%t(W*X) / (W%*%t(W)) 
    
    #  eigenvalues (in decreasing order) and corresponding eigenvectors (in columns) 
    EV <- eigen(S)
    
    #----------------------------------------------------------------
    # step 2: loop through ranks to compute true explained variance
    #----------------------------------------------------------------
    Chi2 <- vector(length=d) # explained variance
    
    for (rank in 1:d){
      # matrix of principal components (new basis)
      P <- EV$vectors[, 1:rank]
      
      # matrix of coefficients (coordinates in new basis)
      C <- matrix(data=NA, nrow=rank, ncol=n)
      for (i in 1:n){
        w2 <- diag(W[, i]^2) 
        C[, i] <- solve(t(P)%*%w2%*%P, t(P)%*%w2%*%X[, i])
      }
      
      # matrix of projections of old data (PC approx X)
      PC <- P%*%C
      
      # explained variance
      Chi2[rank] <- sum(sum((W*PC)^2)) / sum(sum((W*X)^2))
    }
    
    #----------------------------------------------------------------
    # step 3: decide for a rank based on the explained variance 
    #         and compute data for output (same steps as above)
    #----------------------------------------------------------------
    rank <- rankn
    P <- EV$vectors[, 1:rank]
    C <- matrix(data=NA, nrow=rank, ncol=n)
    for (i in 1:n){
      w2 <- diag(W[,i]^2) 
      C[, i] <- solve(t(P)%*%w2%*%P, t(P)%*%w2%*%X[, i])
    }
    PC <- P%*%C
    
    
    
    #----------------------------------------------------------------
    # step 4: plot results and check relationships with traits 
    #         
    #----------------------------------------------------------------
    
    print(Chi2) # explained variance
    print(diff(Chi2))
    
    return(list(coords = t(C),loadings = as.data.frame(P[,1:rankn])))
}    


